/*
* This file is part of TAP2WAV
*
* Copyright (C) 2003 Pavel Cejka
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VERSION         "0.1alpha"
#define OUTPUTFILE	"zxscreen.scr"	/* default filename */

// globln parametry
int     force = 0;
int     verbose = 0;

unsigned char picture_bmp [6206];
unsigned char picture_scr [6144];

int readfile (char* source_filename)
{
        FILE *inputfile;
	
	if (verbose >1) printf ("Opening source file.\n");
	inputfile = fopen (source_filename,"r");

        if (inputfile == NULL)
        {
                printf ("Error: Can't open input file %s.\n",source_filename);
                exit (-1);
        }

	if (verbose >1) printf ("Reading source file.\n");
	
	if (fread (&picture_bmp, 6206, 1, inputfile) == 1)
        {
		if (verbose >1) printf ("Source file readed.\n");
	}
        else
        {
                printf ("Error: Reading from file %s failed.\n",source_filename);
                close (inputfile);
                exit (-1);
        }

        // data ze souboru BMP natena, provm jestli m sprvn formt a zkonvertuju
	close (inputfile);
}

int writefile (char* target_filename)
{
        FILE *outputfile;
	
	if (verbose >1) printf ("Opening target file.\n");
	outputfile = fopen (target_filename,"w+");

        if (outputfile == NULL)
        {
                printf ("Error: Can't open output file %s.\n",target_filename);
                exit (-1);
        }

	if (verbose >1) printf ("Writing data to target file.\n");
	
        if (fwrite (&picture_scr, 6144, 1, outputfile) != 1)          // ZPIS
        {
                printf ("Track writing in %s failed.\n", target_filename);
                fclose (outputfile);
                exit (-1);
        }
        if (verbose) printf ("File succesfully writed.\n");

        // zkonvertovan data uloena do SCR souboru, mono nast do ZX Spectra
	close (outputfile);
}

int convert ()
{
	int	bmpindex, line, zxi1, zxi2, zxi3, i;

	if (verbose >1) printf ("Converting data.\n");

	bmpindex = 6205;
	zxi1 = 0;
	zxi2 = 0;
	zxi3 = 0;
	
	for (line = 0; line < 191; line++)
	{
	    for (i = 31; i >= 0; i--)
	    {
		picture_scr[zxi1+zxi2+zxi3+i] = (255^picture_bmp[bmpindex]);
		bmpindex--;
	    }
	    // nhrada downhl
	    zxi1 += 256;		// o mikrodek dol

	    if (zxi1 == 2048)
	    {
		zxi1 = 0;
		zxi2 += 32;		// o dek dol (8 mikrodk)
		if (zxi2 == 256)
		{
		    zxi2 = 0;
		    zxi3 += 2048;	// o 1/3 obrazovky dol
		}
	    }
	}
}

void version ()
{
        printf ("%s \n", VERSION);
}

void help ()
{
	printf ("BMP2SCR is a special utility converts BW BMPs to SCR for ZX Spectrum.\n");
	printf ("Usage: BMP2SCR [OPTIONS] [SOURCE]\n");
	printf ("[SOURCE] must be BMP picture in resolution 256x192 pixels and 1bpp color depth\n");
	printf ("Output will normal ZX Spectrum screen in same resolution (6144 bytes)\n");
	printf ("Default output is file \"zxscreen.scr\".\n");
	printf ("  -V, --verbose    increase verbosity level\n");
	printf ("  -h, --help       this text\n");
	printf ("  -v, --version    version\n");
}

int main(int argc, char *argv[])
{
	char *source_filename;
	char *target_filename;
	int counter;
	int switches;
	int correct;

	// vstupn soubor je to co nen jinm parametrem, vstupn soubor nen
	// printf ("DEBUG: argc - %i\n", argc);

	switches = 1;
	for (counter=1; counter<argc; counter++)
	{
//		printf ("++ %i ++ %s \n",counter, argv[counter]);
		correct = 0;

		if (!strcmp(argv[counter], "-h") || (!strcmp(argv[counter], "--help")))
		{
			help ();
			switches++;
			correct = 1;
			return (0);
		}
		if (!strcmp(argv[counter], "-v") || (!strcmp(argv[counter], "--version")))
		{
			version ();
			switches++;
			correct = 1;
			return (0);
		}
		if (!strcmp(argv[counter], "-V") || (!strcmp(argv[counter], "--verbose")))
		{
			verbose++;
			switches++;
			correct = 1;
		}
		if (!strcmp(argv[counter], "-f") || (!strcmp(argv[counter], "--force")))
		{
			// bude pepisovat cl
			force = 1;
			switches++;
			correct = 1;
		}
                if ((correct==0) && !((counter==(argc-1)) || (counter==(argc-2))))
                {
                        // posledn parametr jmno cle
                        // pedposledn parametr jmno zdroje
                        printf ("ERROR: unknown parametr - \"%s\"\n", argv[counter]);
                        help ();
                        return (-1);
                }
	}

        source_filename=argv[argc-2];
        target_filename=argv[argc-1];

	if (verbose > 0) printf ("[SOURCE] %s -> [TARGET] %s \n", source_filename, target_filename);

	if (switches!=argc-2)
	{
		printf ("ERROR: Any filename is missing.\n");
		return (-1);
	}
	else
	{
		readfile (source_filename);
		convert ();
		writefile (target_filename);
	}
}
